;*---------------------------------------------------------------------------
;  :Program.	BOND.asm
;  :Contents.	Decruncher for XFDMaster.Library
;  :Author.	WEPL
;  :History.	V 0.9  10.09.95
;		V 0.10 16.10.95 little changes, reduced size
;		V 0.11 07.01.96 optimized some bytes, ID-string changed
;		0.12	13.01.96 freeing memory if dec failed
;  :Copyright.	Public Domain
;  :Language.	68000 Assembler
;  :Translator.	Barfly 1.130
;---------------------------------------------------------------------------*
;##########################################################################

	INCDIR	Includes:
	INCLUDE	lvo/exec.i
	INCLUDE	libraries/xfdmaster.i

;##########################################################################
;----------------------------------------
; XFD-Library-Header

		moveq	#-1,d0
		rts	

		dc.l	XFDF_ID		;id
		dc.w	XFDF_VERSION	;version
		dc.w	0
		dc.l	0,0		;private
		dc.l	_xfdSlave1	;first slave

;----------------------------------------
; version string

		dc.b	'$VER: BOND 0.12 '
	DOSCMD	"WDate >T:date"
	INCBIN	T:date
		dc.b	' by WEPL for xfdmaster.library',0
	CNOP 0,4

;----------------------------------------
; xfdSlave-structure

_xfdSlave1	dc.l	0		; next slave
		dc.w	XFDS_VERSION	; structure version
		dc.w	33		; req xfdmaster version
		dc.l	_name1		; cruncher name
		dc.w	XFDPFF_DATA	; cruncher flags
		dc.w	0		; max length of special info (passwd)
		dc.l	_check1		; check routine
		dc.l	_decrunch1	; decrunch routine
		dc.l	0		; segment recog routine
		dc.l	0		; segment decrunch routine

;----------------------------------------
; name of packer

_name1		dc.b	'BOND Cruncher',0
	EVEN

;----------------------------------------
; code to check crunched
; bergabe :	D0 = ULONG data size
;		A0 = APTR  data
; Rckgabe :	D0 = BOOL  success

_check1		moveq	#24,d1		;minimum 24 bytes
		cmp.l	d1,d0
		blo	.no
		cmpi.l	#"BOND",(-5,a0,d0.l)
		beq	.yes
.no		moveq	#0,d0
		rts

.yes		moveq	#-1,d0
		rts

;----------------------------------------
; code to decrunch the file
; bergabe :	A0 = APTR xfdBufferInfo
; Rckgabe :	D0 = BOOL success

_decrunch1	movem.l	a4-a6,-(a7)
		move.l	a0,a5				;A5 = xfdbi
		move.l	(xfdbi_SourceBuffer,a5),a4	;A4 = source
		move.l	(a4),d0				;size packed
		cmp.l	(xfdbi_SourceBufLen,a5),d0
		bhs	.toshort
		moveq	#3,d1
		and.w	d0,d1				;must LongWord aligned
		bne	.notaligned
		move.l	(-4,a4,d0.l),d0
		move.l	d0,(xfdbi_TargetBufLen,a5)
		move.l	d0,(xfdbi_TargetBufSaveLen,a5)
		move.l	(xfdbi_TargetBufMemType,a5),d1
		move.l	(4).w,a6
		jsr	(_LVOAllocMem,a6)
		move.l	d0,(xfdbi_TargetBuffer,a5)
		bne	.memok
		moveq	#XFDERR_NOMEMORY,d1
		bra	.quit
		
.memok		move.l	d0,a1
		move.l	a4,a0
		bsr	_BOND
		tst.l	d0
		bne	.decok
		move.l	(xfdbi_TargetBufLen,a5),d0
		move.l	(xfdbi_TargetBuffer,a5),a1
		jsr	(_LVOFreeMem,a6)
.notaligned
.toshort	moveq	#0,d0
		moveq	#XFDERR_CORRUPTEDDATA,d1
		bra	.quit
		
.decok		moveq	#0,d1		;errcode
		moveq	#-1,d0

.quit		move.w	d1,(xfdbi_Error,a5)
		movem.l	(a7)+,a4-a6
		rts	

;----------------------------------------
;	Shadow of the Beast
;	Robocop
;
; Format :	$0	dc.l	size packed (without ID-string)
; 		$4	ds.b	data
;			...
;			dc.l	saved reg D0
;			dc.l	chksum
;			dc.l	size unpacked
;	_sizepacked	dc.b	ID-String
;
; bergabe :	A0 = APTR source
;		A1 = APTR destination
; Rckgabe :	D0 = BOOL success

ChkSrc	MACRO
		cmp.l	a0,a3
		beq	_fail
	ENDM
ChkDst	MACRO
		cmp.l	a1,a2
		beq	_fail
	ENDM

_BOND		movem.l	d2-d5/a2-a4,-(a7)
		move.l	a7,a4			;A4 = saved sp
		move.l	a0,a3			;A3 = start of buffer
		ADDA.L	(A0),A0			;A0 = end of buffer
		MOVEA.L	-(A0),A2
		ADDA.L	A1,A2			;A2 = end of target
		MOVE.L	-(A0),D5		;D5 = chksum
		MOVE.L	-(A0),D0
		EOR.L	D0,D5

L_148C		bsr	_getbit
		BCS.B	L_14C6
		MOVEQ	#8,D1
		MOVEQ	#1,D3
		bsr	_getbit
		BCS.B	L_14EC
		MOVEQ	#3,D1
		moveq	#0,d4
L_14A4		BSR.B	_getD2
		MOVE.W	D2,D3
		ADD.W	D4,D3
L_14AA		MOVEQ	#7,D1
L_14AC		bsr	_getbit
		addx.l	d2,d2
		DBRA	D1,L_14AC
	ChkDst
		MOVE.B	D2,-(A2)
		DBRA	D3,L_14AA
		BRA.B	L_14F8

L_14C0		MOVEQ	#8,D1
		MOVEQ	#8,D4
		BRA.B	L_14A4

L_14C6		MOVEQ	#2,D1
		BSR.B	_getD2
		CMP.B	#2,D2
		BLT.B	L_14E2
		CMP.B	#3,D2
		BEQ.B	L_14C0
		MOVEQ	#8,D1
		BSR.B	_getD2
		MOVE.W	D2,D3
		MOVEq	#$000C,D1
		BRA.B	L_14EC

L_14E2		MOVEq	#9,D1
		ADD.W	D2,D1
		ADDQ.W	#2,D2
		MOVE.W	D2,D3
L_14EC		BSR.B	_getD2
L_14EE	ChkDst
		MOVE.B	(-1,A2,D2.W),-(A2)
		DBRA	D3,L_14EE
L_14F8		CMPA.L	A2,A1
		BLT.W	L_148C
		bra	_end

_newD0	ChkSrc
		MOVE.L	-(A0),D0
		EOR.L	D0,D5
		MOVE	#$0010,CCR
		ROXR.L	#1,D0
		RTS

_getbit		lsr.l	#1,d0
		bne	.rts
		bsr	_newD0
.rts		rts

_getD2		SUBQ.W	#1,D1
		moveq	#0,d2
.1514		bsr	_getbit
		addx.l	d2,d2
		DBRA	D1,.1514
		RTS	

_end		moveq	#-1,d0
		tst.l	d5		;chksum
		beq	_ok
_fail		moveq	#0,d0
_ok		move.l	a4,a7
		movem.l	(a7)+,d2-d5/a2-a4
		RTS	

;##########################################################################

	END

