;*---------------------------------------------------------------------------
;  :Program.    Ice_TSM.asm
;  :Contents.   Decruncher for XFDMaster.Library
;  :Author.     WEPL
;  :History.	1.2	11.01.96 imported from TSM.asm + Ice.asm
;				 memory protection read/write
;		1.3	16.01.96 separate _check for each slave
;  :Copyright.  Public Domain
;  :Language.   68000 Assembler
;  :Translator. Barfly 1.130
;---------------------------------------------------------------------------*
;##########################################################################

	INCDIR	Includes:
	INCLUDE	lvo/exec.i
	INCLUDE	libraries/xfdmaster.i

;##########################################################################

;----------------------------------------
; XFD-Library-Header

		moveq	#-1,d0
		rts	

		dc.l	XFDF_ID		;id
		dc.w	XFDF_VERSION	;version
		dc.w	0
		dc.l	0,0		;private
		dc.l	_xfdSlave1	;first slave

;----------------------------------------
; version string

		dc.b	'$VER: Ice_TSM 1.3 '
	DOSCMD	"WDate >T:date"
	INCBIN	T:date
		dc.b	' by WEPL for xfdmaster.library',0
	CNOP 0,4

;----------------------------------------
; xfdSlave-structure

_xfdSlave1	dc.l	_xfdSlave2	; next slave
		dc.w	XFDS_VERSION	; structure version
		dc.w	33		; req xfdmaster version
		dc.l	_name1		; cruncher name
		dc.w	XFDPFF_DATA	; cruncher flags
		dc.w	0		; max length of special info (passwd)
		dc.l	_check1		; check routine
		dc.l	_decrunch1	; decrunch routine
		dc.l	0		; segment recog routine
		dc.l	0		; segment decrunch routine

_xfdSlave2	dc.l	0		; next slave
		dc.w	XFDS_VERSION	; structure version
		dc.w	33		; req xfdmaster version
		dc.l	_name2		; cruncher name
		dc.w	XFDPFF_DATA	; cruncher flags
		dc.w	0		; max length of special info (passwd)
		dc.l	_check2		; check routine
		dc.l	_decrunch1	; decrunch routine
		dc.l	0		; segment recog routine
		dc.l	0		; segment decrunch routine

;----------------------------------------
; name of packer

_name1		dc.b	'TSM! Cruncher',0
_name2		dc.b	'Ice! Cruncher',0
	EVEN

;----------------------------------------
; code to check crunched
; bergabe :	D0 = ULONG data size
;		A0 = APTR  data
; Rckgabe :	D0 = BOOL  success

_check1		cmpi.l	#"TSM!",(a0)
		bne	_no
		bra	_both

_check2		cmpi.l	#"Ice!",(a0)
		bne	_no

_both		moveq	#16,d1		;minimum 12 bytes
		cmp.l	d1,d0
		blo	_no
		moveq	#-1,d0
		rts

_no		moveq	#0,d0
		rts

;----------------------------------------
; code to decrunch the file
; bergabe :	A0 = APTR xfdBufferInfo
; Rckgabe :	D0 = BOOL success

_decrunch1	movem.l	a4-a6,-(a7)
		move.l	a0,a5				;A5 = xfdbi
		move.l	(xfdbi_SourceBuffer,a5),a4	;A4 = source
		move.l	(8,a4),d0
		move.l	d0,(xfdbi_TargetBufLen,a5)
		move.l	d0,(xfdbi_TargetBufSaveLen,a5)
		move.l	(xfdbi_TargetBufMemType,a5),d1
		move.l	(4).w,a6
		jsr	(_LVOAllocMem,a6)
		move.l	d0,(xfdbi_TargetBuffer,a5)
		bne	.memok
		moveq	#XFDERR_NOMEMORY,d1
		bra	.quit
		
.memok		move.l	d0,a1
		move.l	a4,a0
		move.l	(xfdbi_SourceBufLen,a5),d0
		bsr	_TSM
		bne	.decok
		move.l	(xfdbi_TargetBufLen,a5),d0
		move.l	(xfdbi_TargetBuffer,a5),a1
		jsr	(_LVOFreeMem,a6)
		moveq	#0,d0
		moveq	#XFDERR_CORRUPTEDDATA,d1
		bra	.quit
		
.decok		moveq	#-1,d0
		moveq	#0,d1

.quit		move.w	d1,(xfdbi_Error,a5)
		movem.l	(a7)+,a4-a6
		rts

;----------------------------------------
; TSM!		Oscar
; Ice!		Terminator 2
;		HeroQuest
;		Pang!
;
; Format :	$0	dc.l	"TSM!"
;		$4	ULONG	size packed
;		$8	ULONG	size unpacked
;		$C		Daten...
;
; bergabe :	D0 = ULONG source length
;		A0 = APTR source
;		A1 = APTR destination
; Rckgabe :	D0 = BOOL success

_TSM		MOVEM.L	D2-d7/a2-A6,-(A7)
		move.l	a7,a3			;A3 = saved stackpointer
		moveq	#8,d6			;D6 = global #8

		addq.l	#4,a0
		move.l	(a0)+,d1
		cmp.l	d0,d1
		blo	.fail
		LEA	(-8,A0,D1.L),A5		;a5 end of packed-file
	
		move.l	a1,a4			;A4 start destination
		move.l	a1,a6
		add.l	(a0)+,a6		;a6 end of dest
	
		moveq	#0,d7
		bsr	.getbit
		roxr.l	#1,d7
		BSR.B	.8E4C
		cmp.l	a0,a5			;all data read ?
		bne	.fail
		moveq	#-1,d0

	IFD	TERMINATOR
		BSR.W	_GetBit
		BCC.B	L_76
		MOVEA.L	A3,A1
		BSR.W	L_20E
L_76
	ENDC

.end		move.l	a3,a7
		MOVEM.L	(A7)+,D2-d7/a2-A6
		RTS

.fail		moveq	#0,d0
		bra	.end

ChkDest	MACRO				;check for writing outside dest
		cmp.l	a4,a6
		beq	.fail
	ENDM
ChkSrc	MACRO				;check for reading outside source
		cmp.l	a0,a5
		bls	.fail
	ENDM

.8E4C		BSR.B	.getbit
		BCC.B	.8E72
		MOVEQ	#0,D1
		BSR.B	.getbit
		BCC.B	.8E6C
		LEA	.8F50(PC),A1
		MOVEQ	#4,D3
.8E5C		MOVE.L	-(A1),D0
		BSR.B	.getD1
		SWAP	D0
		CMP.W	D0,D1
		DBNE	D3,.8E5C
		ADD.w	(22,A1),D1
.8E6C	ChkDest
	ChkSrc
		MOVE.B	-(A5),-(A6)		;a5 end of source
		DBRA	D1,.8E6C		;a6 end of dest
.8E72		CMPA.L	A4,A6			;a4 begin of dest
		BGT.B	.8ED8
		RTS

.getbit		ADD.L	D7,D7
		bne	.no
.08E84	ChkSrc
		move.b	-(a5),d7
		ror.l	d6,d7
		move.b	-(a5),d7
		ror.l	d6,d7
		move.b	-(a5),d7
		ror.l	d6,d7
		move.b	-(a5),d7
		ror.l	d6,d7
		or	#16,ccr
		addx.l	d7,d7
.no		rts

.getD1		MOVEQ	#0,D1
.8ECC		ADD.L	D7,D7
		bne	.8ED0
		bsr	.getbit
.8ED0		ADDX.w	D1,D1
		DBRA	D0,.8ECC
		RTS

.8ED8		LEA	.8F64(PC),A1
		MOVEQ	#3,D2
.8EDE		BSR.B	.getbit
		DBCC	D2,.8EDE
		MOVEQ	#0,D4
		MOVEQ	#0,D1
		MOVE.B	1(A1,D2.W),D0
		EXT.W	D0
		BMI.B	.8EF2
		BSR.B	.getD1
.8EF2		MOVE.B	6(A1,D2.W),D4
		ADD.W	D1,D4
		BEQ.B	.8F18
		LEA	.8F6E(PC),A1
		MOVEQ	#1,D2
.8F00		BSR.B	.getbit
		DBCC	D2,.8F00
		MOVEQ	#0,D1
		MOVE.B	1(A1,D2.W),D0
		EXT.W	D0
		BSR.B	.getD1
		ADD.W	D2,D2
		ADD.W	6(A1,D2.W),D1
		BRA.B	.8F2A

.8F18		MOVEQ	#0,D1
		MOVEQ	#5,D0
		MOVEQ	#0,D2
		BSR.B	.getbit
		BCC.B	.8F26
		MOVEQ	#8,D0
		MOVEQ	#$0040,D2
.8F26		BSR.B	.getD1
		ADD.W	D2,D1
.8F2A		LEA	2(A6,D4.W),A1
		ADDA.W	D1,A1
	ChkDest
		MOVE.B	-(A1),-(A6)
.8F32	ChkDest
		MOVE.B	-(A1),-(A6)
		DBRA	D4,.8F32
		BRA.W	.8E4C

	CNOP 0,4
		DC.L	$7FFF000E
		DC.L	$FF0007
		DC.L	$70002
		DC.L	$30001
		DC.L	$30001
.8F50		DC.L	$10D
		DC.L	$E
		DC.L	7
		DC.L	4
		DC.L	1

.8F64		DC.B	9
		DC.B	1
		DC.B	0
		DC.B	-1
		DC.B	-1
		DC.B	8
		DC.B	4
		DC.B	2
		DC.B	1
		DC.B	0
.8F6E		DC.B	$B
		DC.B	4
		DC.B	7
		DC.B	0
		DC.B	1
		DC.B	$20
		DC.B	0
		DC.B	0
		DC.B	0
		DC.B	$20

	IFD	TERMINATOR
	;zustzliche transformierung ???
	;32000 Byte Pic 320x200x4
	; A1 = end of dest
L_20E		MOVE.W	#$0F9F,D7	;4000
L_212		MOVEQ	#3,D6		;4x2byte = 8 byte
L_214		MOVE.W	-(A1),D4
		MOVEQ	#3,D5		;4x4 bit = 16 bit
L_218	;move.w d4,$dff180
		ADD.W	D4,D4
		ADDX.w	D0,D0
		ADD.W	D4,D4
		ADDX.w	D1,D1
		ADD.W	D4,D4
		ADDX.w	D2,D2
		ADD.W	D4,D4
		ADDX.w	D3,D3
		DBRA	D5,L_218
		DBRA	D6,L_214
	cmp.l	a1,a4
	bhs	_failure
		MOVEM.W	D0-D3,(A1)
		DBRA	D7,L_212
		RTS	
	ENDC

;##########################################################################

	END

