;*---------------------------------------------------------------------------
;  :Program.    SSUR.asm
;  :Contents.   Decruncher for XFDMaster.Library
;  :Author.     WEPL
;  :History.    0.9	06-Nov-94
;		1.0	19-Nov-94 angepat auf orginal includes
;		1.1	07.01.96 _decrunch optimized
;				 _SSUR partiell rewritten
;				 now secure for reading outside source
;						writing outside destination
;				 buffer will freed if dec failed
;			10.01.96 further optimizes
;  :Copyright.  Public Domain
;  :Language.   68000 Assembler
;  :Translator. Barfly 1.130
;---------------------------------------------------------------------------*
;##########################################################################

	INCDIR	Includes:
	INCLUDE	lvo/exec.i
	INCLUDE	libraries/xfdmaster.i

;##########################################################################

;----------------------------------------
; XFD-Library-Header

		moveq	#-1,d0
		rts	

		dc.l	XFDF_ID		;id
		dc.w	XFDF_VERSION	;version
		dc.w	0
		dc.l	0,0		;private
		dc.l	_xfdSlave1	;first slave

;----------------------------------------
; version string

		dc.b	'$VER: SSUR 1.1 '
	DOSCMD	"WDate >T:date"
	INCBIN	T:date
		dc.b	' by WEPL for xfdmaster.library',0
	EVEN

;----------------------------------------
; xfdSlave-structure

_xfdSlave1	dc.l	0		; next slave
		dc.w	XFDS_VERSION	; structure version
		dc.w	33		; req xfdmaster version
		dc.l	_name1		; cruncher name
		dc.w	XFDPFF_DATA	; cruncher flags
		dc.w	0		; max length of special info (passwd)
		dc.l	_check1		; check routine
		dc.l	_decrunch1	; decrunch routine
		dc.l	0		; segment recog routine
		dc.l	0		; segment decrunch routine

;----------------------------------------
; name of packer

_name1		dc.b	'SSUR Cruncher',0
	EVEN

;----------------------------------------
; code to check crunched
; bergabe :	D0 = ULONG data size
;		A0 = APTR  data
; Rckgabe :	D0 = BOOL  success

_check1		moveq	#12,d1		;minimum 12 bytes
		cmp.l	d1,d0
		blo	.no
		cmpi.l	#"SSUR",(a0)
		beq	.yes
.no		moveq	#0,d0
		rts

.yes		moveq	#-1,d0
		rts

;----------------------------------------
; code to decrunch the file
; bergabe :	A0 = APTR xfdBufferInfo
; Rckgabe :	D0 = BOOL success

_decrunch1	movem.l	a4-a6,-(a7)
		move.l	a0,a5				;A5 = xfdbi
		move.l	(xfdbi_SourceBuffer,a5),a4	;A4 = source
		move.l	(4,a4),d0
		moveq	#8,d1
		ror.w	d1,d0
		swap	d0
		ror.w	d1,d0
		move.l	d0,(xfdbi_TargetBufLen,a5)
		move.l	d0,(xfdbi_TargetBufSaveLen,a5)
		move.l	(xfdbi_TargetBufMemType,a5),d1
		move.l	(4).w,a6
		jsr	(_LVOAllocMem,a6)
		move.l	d0,(xfdbi_TargetBuffer,a5)
		bne	.memok
		moveq	#XFDERR_NOMEMORY,d1
		bra	.quit
		
.memok		move.l	d0,a1
		move.l	a4,a0
		move.l	(xfdbi_SourceBufLen,a5),d0
		bsr	_SSUR
		tst.l	d0
		bne	.decok
		move.l	(xfdbi_TargetBufLen,a5),d0
		move.l	(xfdbi_TargetBuffer,a5),a1
		jsr	(_LVOFreeMem,a6)
		moveq	#0,d0
		moveq	#XFDERR_CORRUPTEDDATA,d1
		bra	.quit
		
.decok		moveq	#-1,d0
		moveq	#0,d1

.quit		move.w	d1,(xfdbi_Error,a5)
		movem.l	(a7)+,a4-a6
		rts

;----------------------------------------
;	Robocop 3
;
; Format :	dc.l	"SSUR"
;		dc.l	size unpacked	little endian
;		dc.l	checksum	little endian
;
; bergabe :	D0 = ULONG source length
;		A0 = APTR source
;		A1 = APTR destination
; Rckgabe :	D0 = BOOL success

_SSUR		movem.l	d2-d7/a2-a4,-(a7)
		move.l	a7,a4		;A4 = saved sp
		moveq	#8,d5		;D5 = #8
		move.l	a0,a3		;A0 = source  (reading a0.l+ )
		add.l	d0,a3		;A3 = end of source
		addq.l	#4,a0		;skip id
		move.l	a1,a2		;A2 = destination start
		bsr	.getlw
		add.l	d7,a1		;A1 = end of destination (writing -a1.b )
		bsr	.getlw
		move.l	d7,d6		;D6 = checksum

		MOVEQ	#0,D2
		bsr	.newd7
		BCS	.1120

.10C0		bsr	.A5
		MOVE.W	D0,D2
		MOVE.W	D0,D3
.10C8
		moveq	#0,D0
		moveq	#7,d4
.lp		bsr	.getbit
		dbf	d4,.lp

	cmp.l	a1,a2
	beq	.error
		MOVE.B	D0,-(A1)
		subq.w	#1,d2
		bne	.10C8
	CMPA.L	A2,A1
	beq	.1136
		NOT.W	D3
		BNE	.1120
.1118		ADD.L	D7,D7
		BNE	.111E
		bsr	.newd7
.111E		BCC	.10C0

.1120		bsr	.A5
		MOVE.W	D0,D2
		bsr	.A5
		MOVE.W	D0,D3
		ADDQ.W	#2,D3
.112A	cmp.l	a1,a2
	beq	.error
		MOVE.B	-1(A1,D2.l),-(A1)
		DBRA	D3,.112A
	CMPA.L	A2,A1
	Bne	.1118
.1136
		moveq	#-1,d0
		tst.l	d6		;checksumm
		beq	.end
.error		moveq	#0,d0
.end		move.l	a4,a7
		movem.l	(a7)+,d2-d7/a2-a4
		RTS

.A5		moveq	#0,D0
		bsr	.getbit
		bsr	.getbit
		move.w	d0,d1
		ADDQ.W	#1,D1
		ADD.W	D1,D1
		ADD.W	D1,D1
		moveq	#0,d0
.115E		bsr	.getbit
		subq.w	#1,d1
		bne	.115E
		RTS	

.getbit		ADD.L	D7,D7
		BNE	.1164
		bsr	.newd7
.1164		ADDX.W	D0,D0
		rts

.newd7		BSR	.getlw
		NOT.L	D7
		SUB.L	D7,d6		;checksum
		ORI.B	#$0010,CCR
		ADDX.L	D7,D7
		RTS	

.getlw		cmp.l	a0,a3		;next read in valid area ?
		bls	.error
		move.l	(a0)+,d7
		ror.w	d5,d7
		swap	d7
		ror.w	d5,d7
		rts

;##########################################################################

	END

